#!/usr/bin/perl
use File::Copy;

# This is a perl script that uses sse to create the CM diagram
# of a set of stars.
# Input: a myriad like data file. only masses are required.
# Output: Snapshots of the stars's masses, Luminocities, Teff for
# different times.

$sse = "./sse";                     # The path to your sse package

$input_file = @ARGV[0];           # The name of the input file
$mass_column = @ARGV[1]-1;          # The column where info about the 
                                  # mass is stored
$M_star = @ARGV[2];		  # The total mass of the cluster in solar 
				  # masses. For converting the N-body mass to 
				  # physical units
$time_end = @ARGV[3];             # Time to end the evolution for every star in Myr

$number_of_snapshots = @ARGV[4];  # Number of equally spaced in time snapshots of the CM diagram.

$separator = @ARGV[5];            # The metallicity changes for particles with names after this point

$metal1 = @ARGV[6];               # The metallicity for the first set of particles
$metal2 = @ARGV[7];               # The metallicity for the second set of particles

$create_ogg = @ARGV[8];           # IF this is 1 then it creates an ogg video of the CM diagram snapshots

open(IN,"$input_file") || die(" Could not open the data file, idiot! \n");

@Particle;                        # Array of hashes to store particle data(mass, id, time)

# Read data and create the Particles. Then close input file
while (<IN>)
    {
	my($new_line) = $_;
        
        if($new_line != m/'#'/)
	  {
	    chomp($new_line); 				# Remove final \n
	    @line = split(/ /, $new_line);		# Split line and get elements stored
	    $name = @line[0];				# First element is the ID of the particle
            $mass = @line[$mass_column];                # This is the mass
            $mass = $mass*$M_star;                      # The mass in solar masses
	    
            if($name < $separator)
              {
                $metallicity = $metal1;
              }
            else
              {
                $metallicity = $metal2;
              }         
            
	    %particle = (name => $name,
			 time => $time,
			 mass => $mass,			# In solar units
			 heavy_metal => $metallicity,   # The metallicity of the star
			 R => $R,			# Radius
			 L => $L,			# Luminocity
			 Teff => $T_eff			# Temperature
			);
	    push @Particle, {%particle}			# Store the particle in the list
	  }
    }

$number_of_stars = scalar(@Particle);
if($separator < $number_of_stars)
  {
   print(" Creating C-M diagram for two clusters \n");
  }
close($input_file);

# Now create the sse input file evolve.in, run sse and create an output file for every star
foreach $part (@Particle)
	{
         # Parameters that are usually set to constant
	 $neta = 0.5;
         $bwind = 0.0;
         $hewind = 0.5;
         $sigma = 0.0;
         $ifflag = 0;
         $wdflag = 1;
         $bhflag = 0;
         $nsflag = 1;
         $mxns = 3.0;
         $idum = 999;
         
         # Time steps for output as fractions of the time that the star spends in each type
         $pts1 = 0.05;
         $pts2 = 0.01;
         $pts3 = 0.02;

         $sse_input = "evolve.in";
         $sse_output = "evolve.dat";

	 # Create the input file for sse
	 open(OUT_in,">$sse_input");	 
         printf OUT_in "%2.4f %2.4f  %2.4f \n", $part->{mass}, $part->{heavy_metal}, $time_end;      
	 printf OUT_in "%2.4f %2.4f  %2.4f %2.4f \n", $neta, $bwind, $hewind, $sigma;
         printf OUT_in "%d %d  %d %d %2.4f %d \n", $ifflag, $wdflag, $bhflag, $nsflag, $mxns, $idum;
         printf OUT_in "%2.4f %2.4f  %2.4f \n", $pts1, $pts2, $pts3; 

	 # Run sse and create output file	
         system("$sse");
	
         if($part->{name} < 10)
             {
               $filename = "0000".$part->{name}.".dat_";
             }
         elsif($part->{name} < 100)
             {
               $filename = "000".$part->{name}.".dat_";
             }
         elsif($part->{name} < 1000)
             {
               $filename = "00".$part->{name}.".dat_";
             }
         elsif($part->{name} < 10000)
             {
               $filename = "0".$part->{name}.".dat_";
             }
         else
             {
               $filename = $part->{name}.".dat_";
             }
         # Give to the output file the appropriate name
	 copy($sse_output,$filename);
	 print " Output file for star $part->{name} created \n";
	}

# Now scan all files and create the files for the CM diagram in different times, which are fractions of the 
# total time.

opendir DIR, "." || die " Cannot open directory\n";
                                         # Open the current directory
@FILES = readdir(DIR);                   # Store the list of files of the directory in an array
closedir(".");
$number_of_files = 0;                    # This is the number of files that will be found
                                         # in the directory

for $file (@FILES)                       # Store only the files that have the "dat_" at the end of their names
{
 if($file =~ m/dat_/)
   {
    chomp($file);
    push @FILES_relevant, $file;         # Create an array with the relevant filenames
                                        
    $number_of_files += 1;               # The total number of filenames should be equal to the total number of stars
   }
}
@FILES = ();                             # Clear memory

@FILES_sorted = sort {$a cmp $b} @FILES_relevant; # Sort the filenames. Adding a ".dat" to everyone

@FILES_relevant = ();                    # Clear memory


# The following does the main job. It goes to every file, finds the data of the star that are closer to the current
# time and stores them into a file. 
# In the end, a list of $number_of_snapshots files will be created, having information about the stars in
# different times. Those files will be used for drawing the C-M diagram.

$dt = $time_end/$number_of_snapshots;
$current_time = 0;
$old_time = 0;
@old_line = ();


while($current_time < $time_end)          # Run until the end of time
{
 # The output snapshot file for the C-M diagram
 $output_file = $current_time.".cm_diagram";
 open(OUTPUT,">$output_file");
 printf OUTPUT "# ID       Time[Myr]    Mass[Msun]    Lumin     Radius    Teff[K]     M_b    M_v      B-V     V-I     Fb      Fv     Fb-Fv   Fv-Fi \n";
 printf OUTPUT "# \n";
 close($output_file);

 # If we have 2 different populations, then another output file will be written
 if($separator < $number_of_stars)
  {
    # The output snapshot file for the C-M diagram
    $output_file2 = $current_time.".cm_diagram2";
    open(OUTPUT2,">$output_file2");
    printf OUTPUT2 "# ID       Time[Myr]    Mass[Msun]    Lumin     Radius    Teff[K]     M_b    M_v      B-V     V-I     Fb      Fv     Fb-Fv   Fv-Fi \n";
    printf OUTPUT2 "# \n";
    close($output_file2);
  }

 print(" Calculating UBV and ACS colors for T = $current_time [Myr]...");
 # Now calculate the colors using Harley's programs
 for $sorted_file (@FILES_sorted)
    {
        $escape_flag = 0;                                                         # If this changes to 1, the loop stops
        #print("Opening $sorted_file \n");
	# Open the files one by one     
        open(IN2,$sorted_file) || die(" Could not open the data file, idiot! \n");# Open file
        $name = $sorted_file;                     
        $name =~ s/.dat//;                                                        # The name of the star, the id
        
	while (<IN2>)
          {
	    my($new_line) = $_;
            $new_line =~ s/\s+/ /g;
            $new_line =~ s/^\s//;
            @line = split(/ /, $new_line);	
            if($line[0] !~ m/T/)
	       {
	         @line = split(/ /, $new_line);		                           # Split line and get elements stored
                 $time = $line[0];
               	 if($time >= $current_time && $escape_flag != 1)                   # Find the time that is closer to the $current_time
	           {

		     if(abs($time-$current_time) > abs($old_time-$current_time))   # Find if previous time is closer to the $current_time
                        {
                         @line = @old_line;                       
                         $time = $old_time;
                        }                  
                     $escape_flag = 1;                                             # Set escape flag to 1 and escape the loop
		     # The output snapshot file for the C-M diagram
                     $output_file = $current_time.".cm_diagram";
		     open(OUTPUT,">>$output_file");

                     $type = int($line[1]);
                     $mass = $line[3];
                     $Luminocity = 10**$line[4];
                     $Radius = 10**$line[5];

		     if($name < $separator)                                        # If we have two populations with different metallicities
                       {
                        $metal = $metal1;
                       }
                     else
                       {
                        $metal = $metal2;
                       }

                     # Calculate the UBV colors                    
                     system("./ubv_colors.exec $metal $type $mass $line[4] $line[5] > ubv.colors");
                     # Calculate the ACS colors
                     system("./acs_colors.exec $metal $type $mass $line[4] $line[5] > acs.colors");
		     # Now extract the information from the files
		     open(UBV,"ubv.colors") || die ("Cannot open ubv.colors\n");
                     while (<UBV>)
                        {
	                  my($ubv_line) = $_;
                          $ubv_line =~ s/\s+/ /g;
                          $ubv_line =~ s/^\s//;
                          @UBV_line = split(/ /, $ubv_line);
                          $UBV_colors{"@UBV_line[0]"} = @UBV_line[1];
                        } 
                     close("ubv.colors");
                     open(ACS,"acs.colors") || die ("Cannot open acs.colors\n");
                     while (<ACS>)
                        {
	                  my($acs_line) = $_;
                          $acs_line =~ s/\s+/ /g;
                          $acs_line =~ s/^\s//;
                          @acs_line = split(/ /, $acs_line);
                          $ACS_colors{"@acs_line[0]"} = @acs_line[1];
                        } 
                     close("acs.colors");
                     $Mb = $UBV_colors{"Mb"};
                     $Mv = $UBV_colors{"Mv"};
                     $b_v = $UBV_colors{"B-V"};
                     $v_i = $UBV_colors{"V-I"};

                     $f555w = $ACS_colors{"F555W"};
                     $f606w = $ACS_colors{"F606W"};
                     $f555w_f606w = $ACS_colors{"F555W-F606W"};
                     $f606w_f814w = $ACS_colors{"F606W-F814W"};
                     # All info is stored to variables

                     # Clear memory
		     @UBV_line = ();
                     @UBV_colors = ();
		     @ACS_line = ();
                     @ACS_colors = ();

                     $T_eff = 10**$line[6];
                     if($name < $separator)                               # If there is no separator only this will be printed
                       {
                         printf OUTPUT "%3d %15.3f %10.5f %15.5f %8.3f %10.3f %7.3f %7.3f %7.3f %7.3f %7.3f %7.3f %7.3f %7.3f\n", $name, $time, $mass, $Luminocity, $Radius, $T_eff, $Mb, $Mv, $b_v, $v_i, $f555w, $f555w,  $f555w_f606w, $f606w_f814w;
                       }
                     else
                       {
                         printf OUTPUT2 "%3d %15.3f %10.5f %15.5f %8.3f %10.3f %7.3f %7.3f %7.3f %7.3f %7.3f %7.3f %7.3f %7.3f\n", $name, $time, $mass, $Luminocity, $Radius, $T_eff, $Mb, $Mv, $b_v, $v_i, $f555w, $f555w,  $f555w_f606w, $f606w_f814w;
                       }
		 }
                $old_time = $time;
                @old_line = @line;
               }
           }
         close($sorted_file);
    }
 close($output_file);
 if($separator < $number_of_stars)
   {
    close($output_file2);
   }

print("...Printing the C-M diagram \n");
 
if($separator >= $number_of_stars)                                        # The case of one population of stars
   {
     $png = $output_file.".png";
     # Now use gnuplot to plot this C-M diagram
     # PNG
     open (GNUPLOT, "|gnuplot");
     reset
     print GNUPLOT <<EOPLOT;
     set term png small xFFFFFF
     set output "$png"
     set nokey
     #set log y
     set xlabel "(B-V)" 
     set ylabel "Mv" 
     set label "T = $current_time Myr" at 1.4,-.2
     plot [-.4:1.7][18:-2] "$output_file" u 9:8 w p
EOPLOT
     close(GNUPLOT);
   }
else									 # The case of two populations of stars
   {
    $png = $output_file.".png";
     # Now use gnuplot to plot this C-M diagram
     # PNG
     open (GNUPLOT, "|gnuplot");
     reset
     print GNUPLOT <<EOPLOT;
     set term png small xFFFFFF
     set output "$png"
     set nokey
     #set log y
     set xlabel "(B-V)" 
     set ylabel "Mv" 
     set label "T = $current_time Myr" at 1.4,-.2
     plot [-.4:1.7][18:-2] "$output_file" u 9:8 w p,  "$output_file2" u 9:8 w p lt 3
EOPLOT
     close(GNUPLOT);
   }

 $current_time = $current_time + $dt;

}

@FILES_sorted = ();                 # Clear memory
# Remove uneccessary files
system("rm -f *dat_");

if($create_ogg == 1)
{
 system("png2theora --video-quality 1 --framerate-numerator 1  --framerate-denominator 1 --keyframe-freq 12 %4d.cm_diagram.png  --output CM_diagram.ogg");
}
